import { DatasourceOptions } from '@genesislcap/foundation-comms';
import { ElementStyles } from '@microsoft/fast-element';
import { ErrorObject } from 'ajv';
import { ErrorTranslator } from '@jsonforms/core';
import { Expression } from '@genesislcap/foundation-criteria';
import { FASTElement } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { JsonFormsState } from '@jsonforms/core';
import { JSONSchema7 } from 'json-schema';
import { Logger } from '@genesislcap/foundation-logger';
import { OwnPropsOfControl } from '@jsonforms/core';
import { RankedTester } from '@jsonforms/core';
import { StatePropsOfControl } from '@jsonforms/core';
import { UISchemaElement } from '@jsonforms/core';
import { ViewTemplate } from '@microsoft/fast-element';

/**
 * Configuration options available for array renderer.
 * @public
 */
export declare type ArrayRendererOptions = {
    /**
     * ChildUiSchema allows you to pass a UiSchema for a particular element,
     * which can then be multiplied many times by the renderer.
     * @public
     */
    childUiSchema?: UiSchema;
};

/**
 * Configuration options available for categorization renderer.
 * @public
 */
export declare type CategorizationRendererOptions = {
    /**
     * childElements allows you to pass the UiSchema array in which the renderers groups will be located
     * @public
     */
    childElements?: UiSchema[];
};

/**
 * Available configuration options for connected combobox/multiselect.
 * @public
 */
export declare type ConnectedRenderersOptions = {
    /**
     * Activates the async search by changing the criteria based on the value from the search-bar.
     * @public
     */
    async?: boolean;
    /**
     * The name of the target Data Server query or Request Server requestReply.
     * @public
     */
    allOptionsResourceName?: string;
    /**
     * A property that is set to configure the resource to which combobox/multiselect is connected.
     * @public
     */
    datasourceConfig?: DatasourceOptions;
    /**
     * Specifies which field should be used as underlying data used when changing selection.
     * @public
     */
    valueField?: string;
    /**
     * Field used to display label for the component.
     * @public
     */
    labelField?: string;
    /**
     * Property to which we can provide the data to be used instead of data from the server.
     * @public
     */
    data?: any[];
};

/**
 * Creates a expressions/criteria from given payload
 * @public
 */
export declare const createExpressions: (payload: any) => Expressions;

declare class DispatchRenderer extends FASTElement {
    jsonforms: any;
    jsonformsChanged(): void;
    props: any;
    propsChanged(): void;
    touched: any;
    submitted: boolean;
    submittedChanged(): void;
    dispatch: any;
    control: any;
    renderer: any;
    prefix: string;
    createRenderer: () => void;
    onBlur(): void;
    get determinedRenderer(): unknown;
    disconnectedCallback(): void;
}

declare type Expressions = {
    options: Expression[];
    numberOptions: Expression[];
    dateOptions: Expression[];
};

/**
 * Foundation filters component for automatically generated filters based on json schema
 * obtained from the api, supplied initial data or supplied JSON schema.
 * Allowing customisable filters elements using UI schema and set of custom renderers
 * @beta
 */
export declare class Filters extends FoundationElement {
    /**
     * Name of the backend resource which will provide metadata
     * used to generate filters
     * @public
     */
    resourceName: string;
    private resourceNameChanged;
    clearFiltersData(): void;
    /**
     * UI schema used to define configuration of the layout and elements in the filters
     * Check {@link UiSchemaElement} for possible options
     *
     * @remarks
     * If not provided will be autogenerated based on json schema or initial data
     *
     * @example
     * Here's a simple example:
     * ```
     * const sampleUISchema = {
     *   type: 'VerticalLayout',
     *   elements: [
     *     {
     *       type: 'Control',
     *       scope: '#/properties/QUANTITY',
     *       label: 'Quantity',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/INSTRUMENT_ID',
     *       options: {
     *         allOptionsResourceName: 'INSTRUMENT',
     *         valueField: 'INSTRUMENT_ID',
     *         labelField: 'INSTRUMENT_ID',
     *       },
     *       label: 'Instrument',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/SIDE',
     *       label: 'Side',
     *     },
     *   ],
     * };
     * ```
     * @public
     */
    uischema: UISchemaElement;
    /**
     * Allows to provide set of renderers used by the filters. If not provided it will default to text-field inputs
     * @public
     */
    renderers: RendererEntry[];
    /**
     * Alternatively to providing {@link Form.resourceName} you can hardcode the JSON schema on the client.
     * @public
     * @remarks
     * Use this when you want to avoid fetching metadata from the server but
     * be aware that it could get out of sync if metadata changes on the server
     */
    jsonSchema: JSONSchema7;
    private connect;
    /**
     * Initial data for the filters
     * @public
     */
    data: any;
    /**
     * Created criteria based on the given data that can be used to filter the data
     * @public
     */
    value: string;
    valueChanged(): void;
    /**
     * @internal
     */
    searchFilters(): void;
    /**
     * @internal
     */
    onChange(event: CustomEvent): void;
    disconnectedCallback(): void;
}

/** @internal */
export declare const filtersLogger: Logger;

/** @public */
export declare const filtersRenderers: any[];

/**
 * Foundation form component for automatically generated forms based on json schema
 * obtained from the api, supplied initial data or supplied JSON schema.
 * Allowing customisable form elements using UI schema and set of custom renderers
 * @beta
 */
export declare class Form extends FoundationElement {
    /**
     * Name of the backend resource which will provide metadata
     * used to generate form and later used for submitting data
     * @public
     */
    resourceName: string;
    private resourceNameChanged;
    private removeDataProperties;
    /**
     * UI schema used to define configuration of the layout and elements in the form
     * Check {@link UiSchemaElement} for possible options
     *
     * @remarks
     * If not provided will be autogenerated based on json schema or initial data
     *
     * @example
     * Here's a simple example:
     * ```
     * const sampleUISchema = {
     *   type: 'VerticalLayout',
     *   elements: [
     *     {
     *       type: 'Control',
     *       scope: '#/properties/QUANTITY',
     *       label: 'Quantity',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/INSTRUMENT_ID',
     *       options: {
     *         allOptionsResourceName: 'INSTRUMENT',
     *         valueField: 'INSTRUMENT_ID',
     *         labelField: 'INSTRUMENT_ID',
     *       },
     *       label: 'Instrument',
     *     },
     *     {
     *       type: 'Control',
     *       scope: '#/properties/SIDE',
     *       label: 'Side',
     *     },
     *   ],
     * };
     * ```
     * @public
     */
    uischema: UISchemaElement;
    /**
     * Allows to provide set of renderers used by the form. If not provided it will default to text-field inputs
     * @public
     */
    renderers: RendererEntry[];
    /**
     * Alternatively to providing {@link Form.resourceName} you can hardcode the JSON schema on the client.
     * @public
     * @remarks
     * Use this when you want to avoid fetching metadata from the server but
     * be aware that it could get out of sync if metadata changes on the server
     */
    jsonSchema: JSONSchema7;
    private errors;
    private connect;
    /**
     * Initial data for the form
     * @public
     */
    data: any;
    /**
     * @internal
     */
    submitting: boolean;
    /**
     * @internal
     */
    submitted: boolean;
    readonly: boolean;
    hideSubmit: boolean;
    /**
     * @internal
     */
    _submit(): Promise<void>;
    get isSubmitHidden(): boolean;
    /**
     * @internal
     */
    submitPart(event: CustomEvent): void;
    /**
     * @internal
     */
    onChange(event: CustomEvent): void;
    disconnectedCallback(): void;
    /**
     * Reset the form state
     * @public
     */
    reset(clearData?: boolean): void;
}

/** @internal */
export declare const foundationFormStyles: ElementStyles;

/** @internal */
export declare const foundationFormTemplate: ViewTemplate<Form, any>;

/** @public */
export declare const genesisErrorTranslator: ErrorTranslator;

/**
 * Configuration options available for group renderer.
 * @public
 */
export declare type GroupRendererOptions = CategorizationRendererOptions;

/** @internal */
export declare const logger: Logger;

/**
 * Ensures the value of the current field (the one the validator is on) matches the one of the target field.
 * @param target - any other field
 * @returns a validator function that will be run by the smart form during the client-side validation stage
 * @public
 */
export declare const mustMatch: (target: string) => (data: any, path: any, label: any) => {
    instancePath: string;
    message: string;
    schemaPath: string;
    keyword: string;
    params: {};
}[];

/** @public */
export declare type RendererEntry = {
    renderer: ViewTemplate<DispatchRenderer, any>;
    tester: RankedTester;
    mapper: (state: JsonFormsState, ownProps: OwnPropsOfControl) => StatePropsOfControl;
};

/** @public */
export declare const renderers: any[];

/**
 * Standard configuration options available for all renderers.
 * @public
 */
export declare type StandardRendererOptions = {
    /**
     * Setting this property allows you to hide the renderer.
     * @public
     */
    hidden?: boolean;
    /**
     * Setting this property enables client-side validation.
     * @public
     */
    validateFn?: (data: any, path: string, label: string) => ErrorObject[];
    i18n?: {
        [key: string]: any;
    };
};

/**
 * Orientation options available for stepper renderer.
 * @public
 */
export declare type StepperOrientationOptions = {
    /**
     * Configuration options available for the stepper renderer which set the renderer horizontally or vertically.
     * @public
     */
    orientation?: StepperOrientationType;
};

/**
 * Available orientation for stepper renderer.
 * @public
 */
export declare type StepperOrientationType = 'vertical' | 'horizontal';

/**
 * Configuration options available for stepper renderer.
 * @public
 */
export declare type StepperRendererOptions = CategorizationRendererOptions;

/**
 * Configuration options available for string renderer.
 * @public
 */
export declare type StringRendererOptions = {
    /**
     * Changes the type attribute in the string renderer from text to password.
     * @public
     */
    isPassword?: boolean;
    /**
     * Setting this property enables text-area instead of text-field.
     * @public
     */
    textarea?: boolean;
};

/**
 * The UI schema, which is passed to JSON Forms, describes the general layout of a form and is just a regular JSON object.
 * It describes the form by means of different UI schema elements, which can often be categorized into either Controls or Layouts.
 * @public
 */
export declare type UiSchema = {
    /**
     * The layouts and control types, which allow you to customize the arrangement of forms.
     * @public
     * */
    type: UiSchemaElementType;
    /**
     * All layouts need to declare an elements property which contains the children which are to be arranged by the layout.
     * It is expected to be an array of UI Schema elements, e.g. controls or other layouts.
     * @public
     */
    elements: UiSchemaElement[];
    /**
     * Allows changing the design-system for renderers.
     * @public
     */
    dsPrefix?: string;
    /**
     * Options for changing the orientation for the stepper renderer.
     * @public
     */
    options?: StepperOrientationOptions;
};

/**
 * A UiSchemaElement that displays property values in a user interface element.
 * @public
 */
export declare type UiSchemaElement = {
    /**
     * The layouts and control types, which allow you to customize the arrangement of forms.
     * @public
     * */
    type: UiSchemaElementType;
    /**
     * The mandatory scope property, which expects a JSON schema reference value, defines to which property of the data the control should be bound to.
     * @public
     */
    scope: string;
    /**
     * By default, controls use the name of property they point to as a label.
     * Setting this property allows you to change the label.
     * @public
     */
    label?: string;
    /**
     * Setting this property allows you to change the placeholder in input renderers.
     * @public
     */
    placeholder?: string;
    /**
     * Optional attribute options that determine how the control is rendered.
     * @public
     */
    options?: UiSchemaElementOptions;
};

/**
 * All configuration options that are available.
 * @public
 */
export declare type UiSchemaElementOptions = ConnectedRenderersOptions | ArrayRendererOptions | CategorizationRendererOptions | GroupRendererOptions | StepperRendererOptions | StringRendererOptions | StandardRendererOptions;

/**
 * The layouts and control types, which allow you to customize the arrangement of forms.
 * @public
 * */
export declare type UiSchemaElementType = 'Control' | 'VerticalLayout' | 'LayoutVertical2Columns' | 'Categorization' | 'Group' | 'Stepper' | 'HorizontalLayout';

export { }
